import nibabel as nib
import numpy as np
import nibabel as nib

def metadata(image_path: str):
    """
    Get metadata of a NIfTI image
    :param image_path: path to NIfTI image
    :return: metadata
    """
    image_info = {}

    # Load the NIfTI file and extract necessary information
    image = nib.load(image_path)
    image_data = image.get_fdata()
    image_header = image.header
    affine = image.affine

    # Get image dimensions and voxel size
    dimensions = image_data.shape
    voxel_size = image_header.get_zooms()[:3]

    # Get spatial and temporal units
    spatial_units, temporal_units = image_header.get_xyzt_units()

    # Get slope and intercept for Hounsfield Unit (HU) conversion
    slope, intercept = image_header.get_slope_inter()[:2]

    # Determine if the image is in HU format
    is_hu_format = slope == 1 and intercept <= 0

    # Check if the image is isotropic
    is_isotropic = all(size == voxel_size[0] for size in voxel_size)

    # Store the information in the dictionary
    image_info["dimensions"] = dimensions
    image_info["voxel_size"] = voxel_size
    image_info["spatial_units"] = spatial_units
    image_info["temporal_units"] = temporal_units
    image_info["slope"] = slope
    image_info["intercept"] = intercept
    image_info["is_hu"] = is_hu_format
    image_info["is_isotropic"] = is_isotropic

    return image_info


# if __name__ == "__main__":
#     image_path = "data/CT.nii.gz"
#     ct_meta = metadata(image_path)
#     print(ct_meta)


import matplotlib.pyplot as plt
from matplotlib.widgets import Slider

from ...plots import assert_shape, generate_mask_colors, plot_contour, plot_image
from ...utils import image_path_to_data_ax, mask_path_to_data_ax


def layered_slider_path(
    image_path,
    masks_path,
    mask_colors=None,
    title_image="Image",
    titles=[],
    cmap="gray",
    origin="upper",
):
    image_data = image_path_to_data_ax(image_path)

    masks_data = [mask_path_to_data_ax(path) for path in masks_path]

    layered_slider_array(
        image_data,
        masks_data,
        mask_colors=mask_colors,
        title_image=title_image,
        titles=titles,
        cmap=cmap,
        origin=origin,
    )


def layered_slider_array(
    image_data,
    masks_data,
    mask_colors=None,
    title_image="Image",
    titles=[],
    cmap="gray",
    origin="upper",
):
    print("Loading images...")

    num_masks = len(masks_data)

    init_slice, last_slice = assert_shape(image_data + masks_data)

    mask_colors = generate_mask_colors(num_masks, mask_colors)

    _, ax = plt.subplots()
    plt.subplots_adjust(bottom=0.25)

    plot_image(ax, image_data[:, :, init_slice], cmap=cmap)

    for i in range(num_masks):
        plot_contour(
            ax,
            masks_data[i][:, :, init_slice],
            color=mask_colors[i],
            # origin=origin,
            levels=[0.5],
        )

    ax.set_xlabel(f"Slice Number: {init_slice}")
    ax.set_title("title")

    slider_ax = plt.axes([0.2, 0.1, 0.6, 0.03])

    slider = Slider(
        slider_ax,
        "Slice",
        0,
        last_slice,
        valinit=init_slice,
        valstep=1,
    )

    def update(val):
        slice_num = int(slider.val)
        ax.clear()

        plot_image(ax, image_data[:, :, slice_num], cmap="gray")
        for i in range(num_masks):
            plot_contour(ax, masks_data[i][:, :, slice_num], color=mask_colors[i])

        ax.set_xlabel(f"Slice Number: {slice_num}")
        ax.set_title("title")

    slider.on_changed(update)

    plt.show()



import SimpleITK as sitk

import matplotlib.pyplot as plt

def read_mha_image(file_path):
    image = sitk.ReadImage(file_path)
    return image

def get_image_properties(image):
    size = image.GetSize()
    origin = image.GetOrigin()
    spacing = image.GetSpacing()
    pixel_data = sitk.GetArrayFromImage(image)
    return size, origin, spacing, pixel_data

if __name__ == "__main__":
    mha_file_path = "/storage/sync/git/mohsen/medviz/dataset/PROV_RectalCA_001_pre_ax_T2_raw.mha"
    # mha_file_path ="/storage/sync/git/mohsen/medviz/dataset/PROV_RectalCA_001_pre_ax_T2_raw_resampled.mha"
    image = read_mha_image(mha_file_path)
    size, origin, spacing, pixel_data = get_image_properties(image)

    print("shape of pixel_data:", pixel_data.shape)
    plt.imshow(pixel_data[10,:,:], cmap='gray')
    plt.show()


    print("Image size:", size)
    print("Image origin:", origin)
    print("Image spacing:", spacing)
    print("Image pixel data shape:", pixel_data.shape)



"""
Medical image reader

Extensions:
    - DICOM
    - NIFTI

"""
from pathlib import Path

import nibabel as nib
import numpy as np
import pydicom as dicom

from .helper_path import path_in


def reader(path: str or Path) -> nib.Nifti1Image or dicom.dataset.FileDataset:
    path = path_in(path)
    if path.suffix == ".nii" or path.suffix == ".nii.gz":
        return nib.load(path)
    elif path.suffix == ".dcm":
        return dicom.dcmread(path)
    else:
        raise TypeError(f"Unsupported file type: {path.suffix}")


def convert_arr(path: str or Path) -> np.ndarray:
    path = path_in(path)
    if path.suffix == ".nii" or path.suffix == ".nii.gz":
        return nib.load(path).get_fdata()
    elif path.suffix == ".dcm":
        return dicom.dcmread(path).pixel_array
    else:
        raise TypeError(f"Unsupported file type: {path.suffix}")



import SimpleITK as sitk
import numpy as np
from ..utils import path_in, save_path_file

def resample_mha(input_path, output_path, new_voxel_size, method="trilinear"):
    input_path = path_in(input_path)

    # Load the input image using SimpleITK
    img = sitk.ReadImage(input_path)
    data = sitk.GetArrayFromImage(img)
    spacing = img.GetSpacing()

    print("Current voxel size", spacing)
    current_voxel_size = np.array(spacing)

    scaling_factors = current_voxel_size / new_voxel_size
    print("Scaling factors", scaling_factors)

    # Compute the new shape after resampling
    print("Input shape", data.shape)
    new_shape = np.ceil(data.shape * scaling_factors).astype(int)
    print("Output shape", new_shape)

    # Use Trilinear interpolation to resample the data
    resampled_data = sitk.Resample(img, new_shape.tolist(), sitk.Transform(), sitk.sitkLinear)

    # Create a new SimpleITK image with resampled data and updated spacing
    resampled_img = sitk.GetArrayFromImage(resampled_data)
    resampled_spacing = new_voxel_size
    resampled_img = sitk.GetImageFromArray(resampled_img)
    resampled_img.SetSpacing(resampled_spacing)

    # Save the resampled image to the output path
    save_path = save_path_file(output_path, suffix=".mha")

    sitk.WriteImage(resampled_img, save_path)
